
 #define DT_DRV_COMPAT sitronix_st7305

 #include "display_st7305.h"
 //#include "pic.h"
 
 #include <zephyr/device.h>
 #include <zephyr/drivers/mipi_dbi.h>
 #include <zephyr/pm/device.h>
 #include <zephyr/sys/byteorder.h>
 #include <zephyr/drivers/display.h>
 
 #define LOG_LEVEL CONFIG_DISPLAY_LOG_LEVEL
 #include <zephyr/logging/log.h>
 LOG_MODULE_REGISTER(display_st7305);
 
 #define DISPLAY_WIDTH  250
 #define BUFFER_WIDTH   256// Due to 12-bit alignment
 #define DISPLAY_HEIGHT 132
 #define PAGE_HEIGHT    2 // Each page contains two rows
 #define NUM_PAGES      (DISPLAY_HEIGHT / PAGE_HEIGHT)
 #define NUM_COLUMNS    (BUFFER_WIDTH / 12) // 132 / 12 = 11
 #define DATA_BYTES     3                   // Each column requires 3 bytes (24-bit data) // Horizontal Writing
     

 struct st7305_config {
	 const struct device *mipi_dbi;
	 const struct mipi_dbi_config dbi_config;
	 uint8_t nvm_param[2]; // Set to [17 02]
	 uint8_t boost_param; // Set to <0x01>
	 uint8_t gvctrl_param[2]; // set to [08 02]
	 uint8_t vshp_param[4]; // Set to [41 41 41 41]
	 uint8_t vslp_param[4]; // [32 32 32 32]
	 uint8_t vshn_param[4]; // [4B 4B 4B 4B]
	 uint8_t vsln_param[4]; // [00 00 00 00]
	 uint8_t osc_param[2]; // [A6 E9]
	 uint8_t frctl_param; // <0x13>
 
	 uint8_t hpmctl_param[10]; // Set to [E5 F6 05 46 77 77 77 77 76 45]
	 uint8_t lpmctl_param[8]; // Set to [05 46 77 77 77 77 76 45]
	 uint8_t gatetm_param[3]; // set to [32 03 1F]
	 uint8_t lpwr_param[3]; // Set to [C1 41 26]
	 uint8_t caset_param[2]; // Set to [19 23]
	 uint8_t raset_param[2]; // set to [00 7C] 
 
	 uint8_t srceq_param; // Set to <0x13>
	 uint8_t gtline_param; // Set to <0x3F>
 
	 uint8_t srcv_param; // Set to <0x00>
	 uint8_t mdac; // set to <0x48>
	 uint8_t dfs_param; // Set to  <0x11>
	 uint8_t pset_param; // Set <0x25>
	 uint8_t enpwd_param; // Set to <0xFF>
	 bool inversion_on; 
	 uint16_t height; // Set to 250
	 uint16_t width; // set to 132
 };
 
 struct st7305_data {
	 uint16_t x_offset;
	 uint16_t y_offset;
 };
 

 static void st7305_set_lcd_margins(const struct device *dev, uint16_t x_offset, uint16_t y_offset)
 {
	 struct st7305_data *data = dev->data;
 
	 data->x_offset = x_offset;
	 data->y_offset = y_offset;
 }
 
 static void st7305_transmit(const struct device *dev, uint8_t cmd, uint8_t *tx_data,
				  size_t tx_count)
 {
	 const struct st7305_config *config = dev->config;
 
	 mipi_dbi_command_write(config->mipi_dbi, &config->dbi_config, cmd, tx_data, tx_count);
 }
 
 static void st7305_exit_sleep(const struct device *dev)
 {
	 st7305_transmit(dev, ST7305_CMD_SLEEP_OUT, NULL, 0);
	 k_sleep(K_MSEC(120));
 }
 
 static void st7305_reset_display(const struct device *dev)
 {
	 const struct st7305_config *config = dev->config;
	 int ret;
 
	 LOG_DBG("Resetting display");
 
	 k_sleep(K_MSEC(1));
	 ret = mipi_dbi_reset(config->mipi_dbi, 6);
	 if (ret == -ENOTSUP) {
		 /* Send software reset command */
		 st7305_transmit(dev, ST7305_CMD_SW_RESET, NULL, 0);
		 k_sleep(K_MSEC(5));
	 } else {
		 k_sleep(K_MSEC(20));
	 }
 }
 
 static int st7305_blanking_on(const struct device *dev)
 {
	 st7305_transmit(dev, ST7305_CMD_DISP_OFF, NULL, 0);
	 return 0;
 }
 
 static int st7305_blanking_off(const struct device *dev)
 {
	 st7305_transmit(dev, ST7305_CMD_DISP_ON, NULL, 0);
	 return 0;
 }
 
 static void st7305_set_mem_area(const struct device *dev, const uint16_t x, const uint16_t y,
				  const uint16_t w, const uint16_t h)
 {
	uint16_t y_start = 25 + (y/12); // Adjust the x range to fit within the CASET range
	uint16_t y_end = y_start + (h/12) - 1;
	uint16_t x_start = 0;
	uint16_t x_end = 124;   

	 // Ensure x_end does not exceed the maximum value of 35
	 if (x_start > 124) {
		//LOG_ERR("Incorrect y_End %d", y_end);
	 	x_start= 124;
	 }

	 // Send CASET (Column Address Set)
	 uint8_t caset[] = {
		 y_start, /* Start column */
		 y_end    /* End column */
	 };
 
	 LOG_ERR("XSTART XEND %dx%d  @ YSTART YEND %dx%d  :   H = %d", x_start, x_end, y_start, y_end,h);
 
	
	 st7305_transmit(dev, ST7305_CMD_CASET, caset, sizeof(caset));
 
	 // Send RASET (Row Address Set)
	 uint8_t raset[] = {
		 x_start,x_end /* End row */
	 };
	 st7305_transmit(dev, ST7305_CMD_RASET, raset, sizeof(raset));
 }  // Horizontal Writing
 
 static void st7305_write_bitmap(const struct device *dev, const uint8_t *src, uint16_t width,
				 uint16_t height, uint8_t *temp_buffer, size_t buf_size)
 {
	 // Clear the temporary buffer
	 memset(temp_buffer, 0, buf_size);

	 // Iterate over each column (11 columns total)
	 for (int col = 0; col < (height/12); col++) {
		if (col >10){
			break;
		}
		 for (int page = 0; page < 125; page++) { // 125 pages, each representing 2 rows
			 uint8_t *data = &temp_buffer[(page * 3) + (col * 375)];

			 // Determine byte and bit offsets based on page and column
			 int byte_index = page / 4; // Which byte (byte 0 or byte 1)
			 int bit_offset_in_byte = (page % 4) * 2; // 0, 2, 4, or 6 bits within the byte

			 for (int i = 0; i < 12; i++) {         // Extract 24 bits in 12 steps
				 int base_byte_offset = i * 32; // Base byte offset in LVGL buffer

				 // Calculate the byte_offset for the LVGL buffer based on page and
				 // column
				 int byte_offset = base_byte_offset + byte_index + (col * 384);

				 if (byte_offset >= ((height*width) / 8)) {
					LOG_ERR("Byte offset out of bounds: %d", byte_offset);
					 continue; // Prevent out-of-bounds access
				 }

				 // Extract the two bits based on the calculated bit_offset
				 uint8_t pixel1 = (src[byte_offset] >> (bit_offset_in_byte)) & 1;
				 uint8_t pixel2 = (src[byte_offset] >> (bit_offset_in_byte + 1)) & 1;

				 // Adjust storage based on LSB-first order
				 int byte_pos = i / 4; // Each byte holds 4 pairs (A, B)
				 int bit_pos = (3-(i % 4));  // Use the lower bits for LSB-first

				 // Store interleaved bits in the correct position
				 data[byte_pos] |=(pixel1<< (bit_pos *2 +1)); // Store pixel1 in the correct bit position
				 data[byte_pos] |=(pixel2 << (bit_pos * 2 )); // Store pixel2 in the next bit position
			 }
		 }
	 }
 } // Horizontal Writing

 
 static int st7305_write(const struct device *dev, const uint16_t x, const uint16_t y,
			 const struct display_buffer_descriptor *desc, const void *buf)
 {
	 struct st7305_data *data = dev->data;
	 const struct st7305_config *config = dev->config;
	 struct display_buffer_descriptor mipi_desc;
	 const uint8_t *write_data_start = (const uint8_t *)buf;
	 enum display_pixel_format pixfmt;
 
	 if (!buf) {
		 LOG_WRN("Display buffer is NULL");
		 return -EINVAL;
	 }
 
	 if ((y + desc->height) > 132) {
		 LOG_ERR("Buffer out of bounds (height)");
		 return -EINVAL;
	 }
 
	 if (x != 0) {
		 LOG_ERR("X-coordinate must be 0");
		 return -EINVAL;
	 }
 
	 LOG_ERR("Incoming Buf size %d", desc->buf_size);
 
	 // Calculate required buffer size
	 
	 LOG_ERR("X : %d Y : %d", x,y);
 
	 // Allocate temporary buffer for conversion
	 //LOG_ERR("TEMP Buf size %d", temp_buf_size);
	 LOG_ERR("height %d", desc->height);
	 LOG_ERR("width. %d", desc->width);

	// k_sleep(K_MSEC(500));

	 if (y>=122)
	 {
		return 0;
	 }
	 
 
	 uint8_t temp_buffer[4125];
	 //LOG_ERR("here1");
	 memset(temp_buffer, 0, sizeof(temp_buffer));
	 //LOG_ERR("here2");
	 // Convert the bitmap to ST7305 format
	 st7305_write_bitmap(dev, write_data_start, desc->width, desc->height,temp_buffer, sizeof(temp_buffer));
 
	 // Set memory area in the ST7305
	 st7305_set_mem_area(dev, x, y, desc->width, desc->height);
 
	 // Configure the MIPI DBI transfer descriptor
	 mipi_desc.height = 12;   // Each page contains two rows
	 mipi_desc.buf_size = 375; // Each column requires 3 bytes (24-bit data)
	 pixfmt = PIXEL_FORMAT_MONO01;
	 mipi_desc.width = 3000; // Each column is 12 pixels wide
	 mipi_desc.pitch = 3000; // Each row must be byte-aligned
	// Horizontal Display

 
	 // Send RAM Write (RAMWR) command
	 st7305_transmit(dev, ST7305_CMD_RAMWR, NULL, 0);
 
	 // Calculate number of columns
	 //uint16_t num_columns = (desc->width / 12) * (desc->height / 2);
	 
	 uint8_t * print_temp_buffer = temp_buffer;
 
	 // Transmit data column by column
	 for (uint16_t col = 0; col < (desc->height /12); ++col) {
		if(col >10){
			break;
		}
		 mipi_dbi_write_display(config->mipi_dbi, &config->dbi_config, print_temp_buffer,
						&mipi_desc, pixfmt);
						print_temp_buffer +=375;
		 //k_sleep(K_MSEC(100));
	 }  // Horizontal Display

 
	 return 0;
 }  // Horizontal Writing

 
 static void st7305_get_capabilities(const struct device *dev,
					  struct display_capabilities *capabilities)
 {
	 const struct st7305_config *config = dev->config;
 
	 memset(capabilities, 0, sizeof(struct display_capabilities));
	 capabilities->x_resolution = config->width;
	 capabilities->y_resolution = config->height;
	 LOG_ERR(" from Get capabilities width %d height %d", config->width, config->height);
	 capabilities->supported_pixel_formats = PIXEL_FORMAT_MONO01;
	 capabilities->current_pixel_format = PIXEL_FORMAT_MONO01;
	 capabilities->screen_info = SCREEN_INFO_X_ALIGNMENT_WIDTH;
	 capabilities->current_orientation = DISPLAY_ORIENTATION_NORMAL;
 }
 
 static int st7305_set_pixel_format(const struct device *dev,
					 const enum display_pixel_format pixel_format)
 {
 
	 if (pixel_format == PIXEL_FORMAT_MONO01) {
 
		 return 0;
	 }
	 LOG_ERR("Pixel format change not implemented");
	 return -ENOTSUP;
 }
 
 static int st7305_set_orientation(const struct device *dev,
					const enum display_orientation orientation)
 {
	 if (orientation == DISPLAY_ORIENTATION_NORMAL) {
		 return 0;
	 }
	 LOG_ERR("Changing display orientation not implemented");
	 return -ENOTSUP;
 }
 
 static void st7305_lcd_init(const struct device *dev)
 {
	 struct st7305_data *data = dev->data;
	 const struct st7305_config *config = dev->config;
	 uint8_t tmp;
 
	 LOG_ERR("Init begin");
 
	 st7305_set_lcd_margins(dev, data->x_offset, data->y_offset);
 
 
	 st7305_transmit(dev, ST7305_CMD_SW_RESET, NULL, 0);
 
	 k_sleep(K_MSEC(125));
 
	 st7305_transmit(dev, ST7305_CMD_NVMCTRL, (uint8_t *)config->nvm_param,
			  sizeof(config->nvm_param));
 
	 tmp = config->boost_param;
	 st7305_transmit(dev, ST7305_CMD_BOOSTER_EN, &tmp, 1);
 
	 st7305_transmit(dev, ST7305_CMD_GVCTRL, (uint8_t *)config->gvctrl_param,
			  sizeof(config->gvctrl_param));
 
	 st7305_transmit(dev, ST7305_CMD_VSHP, (uint8_t *)config->vshp_param,
			  sizeof(config->vshp_param));
 
	 st7305_transmit(dev, ST7305_CMD_VSLP, (uint8_t *)config->vslp_param,
			  sizeof(config->vslp_param));
 
	 st7305_transmit(dev, ST7305_CMD_VSHN, (uint8_t *)config->vshn_param,
			  sizeof(config->vshn_param));
 
	 st7305_transmit(dev, ST7305_CMD_VSLN, (uint8_t *)config->vsln_param,
			  sizeof(config->vsln_param));
 
	 st7305_transmit(dev, ST7305_CMD_OSC, (uint8_t *)config->osc_param,
			  sizeof(config->osc_param));
 
	 tmp = config->frctl_param;
	 st7305_transmit(dev, ST7305_CMD_PORCTRL, &tmp, 1);
 
	 st7305_transmit(dev, ST7305_CMD_HPMCTL, (uint8_t *)config->hpmctl_param,
			  sizeof(config->hpmctl_param));
 
	 st7305_transmit(dev, ST7305_CMD_LPMCTL, (uint8_t *)config->lpmctl_param,
			  sizeof(config->lpmctl_param));
 
	 st7305_transmit(dev, ST7305_CMD_GATE_TIMING, (uint8_t *)config->gatetm_param,
			  sizeof(config->gatetm_param));
 
	 tmp = config->srceq_param;
	 st7305_transmit(dev, ST7305_CMD_SRCEQ, &tmp, 1);
 
	 tmp = config->gtline_param;
	 st7305_transmit(dev, ST7305_CMD_GTLINE, &tmp, 1);
 
	 st7305_transmit(dev, ST7305_CMD_SLEEP_OUT, NULL, 0);
 
	 k_sleep(K_MSEC(120));
 
	 tmp = config->srcv_param;
	 st7305_transmit(dev, ST7305_CMD_SRCV, &tmp, 1);
 
	 /* Memory Data Access Control */
	 tmp = config->mdac;
	 st7305_transmit(dev, ST7305_CMD_MADCTL, &tmp, 1);
 
	 tmp = config->dfs_param;
	 st7305_transmit(dev, ST7305_CMD_DFS, &tmp, 1);
 
	 tmp = config->pset_param;
	 st7305_transmit(dev, ST7305_CMD_PSET, &tmp, 1);
 
	 tmp = config->enpwd_param;
	 st7305_transmit(dev, ST7305_CMD_ENPWD, &tmp, 1);
 
	 st7305_transmit(dev, ST7305_CMD_DISP_ON, NULL, 0);
 
	 st7305_transmit(dev, ST7305_CMD_LPMON, NULL, 0);
 
	 k_sleep(K_MSEC(120));
 
	 st7305_transmit(dev, ST7305_CMD_INV_ON, NULL, 0);
 
	 LOG_ERR("Init end");
 }
 
 static int st7305_init(const struct device *dev)
 {
	 const struct st7305_config *config = dev->config;
 
	 if (!device_is_ready(config->mipi_dbi)) {
		 LOG_ERR("MIPI DBI device not ready");
		 return -ENODEV;
	 }
 
	 st7305_reset_display(dev);
 
	 st7305_blanking_on(dev);
 
	 st7305_lcd_init(dev);
 
	 st7305_exit_sleep(dev);
 
	 return 0;
 }
 
 #ifdef CONFIG_PM_DEVICE
 static int st7305_pm_action(const struct device *dev, enum pm_device_action action)
 {
	 int ret = 0;
 
	 switch (action) {
	 case PM_DEVICE_ACTION_RESUME:
		 st7305_exit_sleep(dev);
		 break;
	 case PM_DEVICE_ACTION_SUSPEND:
		 st7305_transmit(dev, ST7305_CMD_SLEEP_IN, NULL, 0);
		 break;
	 default:
		 ret = -ENOTSUP;
		 break;
	 }
 
	 return ret;
 }
 #endif /* CONFIG_PM_DEVICE */
 
 static const struct display_driver_api st7305_api = {
	 .blanking_on = st7305_blanking_on,
	 .blanking_off = st7305_blanking_off,
	 .write = st7305_write,
	 .get_capabilities = st7305_get_capabilities,
	 .set_pixel_format = st7305_set_pixel_format,
	 .set_orientation = st7305_set_orientation,
 };
 
#define st7305_WORD_SIZE(inst)                                                                    \
	 ((DT_INST_PROP(inst, mipi_mode) == MIPI_DBI_MODE_SPI_4WIRE) ? SPI_WORD_SET(8)              \
									 : SPI_WORD_SET(8))
 #define st7305_INIT(inst)                                                                         \
	 static const struct st7305_config st7305_config_##inst = {                               \
		 .mipi_dbi = DEVICE_DT_GET(DT_INST_PARENT(inst)),                                   \
		 .dbi_config =                                                                      \
			 MIPI_DBI_CONFIG_DT_INST(inst,                                              \
						 st7305_WORD_SIZE(inst) | SPI_TRANSFER_MSB |       \
							 SPI_OP_MODE_MASTER | (0 << 1) | (0 << 2),  \
						 0),                                                \
		 .nvm_param = DT_INST_PROP(inst, nvm_param),                                        \
		 .boost_param = DT_INST_PROP(inst, boost_param),                                    \
		 .gvctrl_param = DT_INST_PROP(inst, gvctrl_param),                                  \
		 .vshp_param = DT_INST_PROP(inst, vshp_param),                                      \
		 .vslp_param = DT_INST_PROP(inst, vslp_param),                                      \
		 .vshn_param = DT_INST_PROP(inst, vshn_param),                                      \
		 .vsln_param = DT_INST_PROP(inst, vsln_param),                                      \
		 .osc_param = DT_INST_PROP(inst, osc_param),                                        \
		 .frctl_param = DT_INST_PROP(inst, frctl_param),                                    \
		 .hpmctl_param = DT_INST_PROP(inst, hpmctl_param),                                  \
		 .lpmctl_param = DT_INST_PROP(inst, lpmctl_param),                                  \
		 .gatetm_param = DT_INST_PROP(inst, gatetm_param),                                  \
		 .lpwr_param = DT_INST_PROP(inst, lpwr_param),                                      \
		 .caset_param = DT_INST_PROP(inst, caset_param),                                    \
		 .raset_param = DT_INST_PROP(inst, raset_param),                                    \
		 .srceq_param = DT_INST_PROP(inst, srceq_param),                                    \
		 .gtline_param = DT_INST_PROP(inst, gtline_param),                                  \
		 .srcv_param = DT_INST_PROP(inst, srcv_param),                                      \
		 .mdac = DT_INST_PROP(inst, mdac),                                                  \
		 .dfs_param = DT_INST_PROP(inst, dfs_param),                                        \
		 .pset_param = DT_INST_PROP(inst, pset_param),                                      \
		 .enpwd_param = DT_INST_PROP(inst, enpwd_param),                                    \
		 .inversion_on = !DT_INST_PROP(inst, inversion_off),                                \
		 .width = DT_INST_PROP(inst, width),                                                \
		 .height = DT_INST_PROP(inst, height),                                              \
	 };                                                                                         \
																									\
	 static struct st7305_data st7305_data_##inst = {                                         \
		 .x_offset = DT_INST_PROP(inst, x_offset),                                          \
		 .y_offset = DT_INST_PROP(inst, y_offset),                                          \
	 };                                                                                         \
																									\
	 PM_DEVICE_DT_INST_DEFINE(inst, st7305_pm_action);                                         \
																									\
	 DEVICE_DT_INST_DEFINE(inst, &st7305_init, PM_DEVICE_DT_INST_GET(inst),                    \
				   &st7305_data_##inst, &st7305_config_##inst, POST_KERNEL,           \
				   CONFIG_DISPLAY_INIT_PRIORITY, &st7305_api);
 
 DT_INST_FOREACH_STATUS_OKAY(st7305_INIT)  